@@ -75,12 +75,19 @@
 //!    }
 //!    assert_eq!(sim, 4);
 //! ```
-use num_traits::{identities::{one,zero}, PrimInt, Unsigned};
+use num_traits::{
+    identities::{one, zero},
+    PrimInt, Unsigned,
+};
 use std::cmp::Ordering::{self};
 use std::collections::VecDeque;
 
+#[cfg(feature = "with_serde")]
+use serde::{Deserialize, Serialize};
+
 /// Represent a range from [start, stop)
 /// Inclusive start, exclusive of stop
+#[cfg_attr(feature = "with_serde", derive(Serialize, Deserialize))]
 #[derive(Eq, Debug, Clone)]
 pub struct Interval<I, T>
 where
@@ -94,6 +101,7 @@ where
 
 /// Primary object of the library. The public intervals holds all the intervals and can be used for
 /// iterating / pulling values out of the tree.
+#[cfg_attr(feature = "with_serde", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone)]
 pub struct Lapper<I, T>
 where
@@ -126,7 +134,7 @@ where
     pub fn intersect(&self, other: &Interval<I, T>) -> I {
         std::cmp::min(self.stop, other.stop)
             .checked_sub(std::cmp::max(&self.start, &other.start))
-            .unwrap_or(zero::<I>())
+            .unwrap_or_else(zero::<I>)
     }
 
     /// Check if two intervals overlap
@@ -143,12 +151,10 @@ where
 {
     #[inline]
     fn cmp(&self, other: &Interval<I, T>) -> Ordering {
-        if self.start < other.start {
-            Ordering::Less
-        } else if other.start < self.start {
-            Ordering::Greater
-        } else {
-            self.stop.cmp(&other.stop)
+        match self.start.cmp(&other.start) {
+            Ordering::Less => Ordering::Less,
+            Ordering::Greater => Ordering::Greater,
+            Ordering::Equal => self.stop.cmp(&other.stop),
         }
     }
 }
@@ -160,7 +166,7 @@ where
 {
     #[inline]
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        Some(self.cmp(&other))
+        Some(self.cmp(other))
     }
 }
 
@@ -200,7 +206,7 @@ where
             let i_len = interval
                 .stop
                 .checked_sub(&interval.start)
-                .unwrap_or(zero::<I>());
+                .unwrap_or_else(zero::<I>);
             if i_len > max_len {
                 max_len = i_len;
             }
@@ -343,9 +349,9 @@ where
         self.max_len = self
             .intervals
             .iter()
-            .map(|x| x.stop.checked_sub(&x.start).unwrap_or(zero::<I>()))
+            .map(|x| x.stop.checked_sub(&x.start).unwrap_or_else(zero::<I>))
             .max()
-            .unwrap_or(zero::<I>());
+            .unwrap_or_else(zero::<I>);
     }
 
     /// Determine the first index that we should start checking for overlaps for via a binary
@@ -545,12 +551,11 @@ where
             first += 1;
         }
         let num_cant_after = len - last;
-        let result = len - first - num_cant_after;
+        len - first - num_cant_after
         //println!("{:#?}", self.starts);
         //println!("{:#?}", self.stops);
         //println!("start found in stops: {}", first);
         //println!("stop found in starts: {}", last);
-        result
     }
 
     /// Find all intervals that overlap start .. stop
@@ -566,7 +571,7 @@ where
         IterFind {
             inner: self,
             off: Self::lower_bound(
-                start.checked_sub(&self.max_len).unwrap_or(zero::<I>()),
+                start.checked_sub(&self.max_len).unwrap_or_else(zero::<I>),
                 &self.intervals,
             ),
             end: self.intervals.len(),
@@ -595,13 +600,14 @@ where
         if *cursor == 0 || (*cursor < self.intervals.len() && self.intervals[*cursor].start > start)
         {
             *cursor = Self::lower_bound(
-                start.checked_sub(&self.max_len).unwrap_or(zero::<I>()),
+                start.checked_sub(&self.max_len).unwrap_or_else(zero::<I>),
                 &self.intervals,
             );
         }
 
         while *cursor + 1 < self.intervals.len()
-            && self.intervals[*cursor + 1].start < start.checked_sub(&self.max_len).unwrap_or(zero::<I>())
+            && self.intervals[*cursor + 1].start
+                < start.checked_sub(&self.max_len).unwrap_or_else(zero::<I>)
         {
             *cursor += 1;
         }
@@ -801,8 +807,7 @@ mod tests {
                 val: 0,
             })
             .collect();
-        let lapper = Lapper::new(data);
-        lapper
+        Lapper::new(data)
     }
     fn setup_overlapping() -> Lapper<usize, u32> {
         let data: Vec<Iv> = (0..100)
@@ -813,8 +818,7 @@ mod tests {
                 val: 0,
             })
             .collect();
-        let lapper = Lapper::new(data);
-        lapper
+        Lapper::new(data)
     }
     fn setup_badlapper() -> Lapper<usize, u32> {
         let data: Vec<Iv> = vec![
@@ -829,8 +833,7 @@ mod tests {
             Iv{start: 68, stop: 71, val: 0}, // overlap start
             Iv{start: 70, stop: 75, val: 0},
         ];
-        let lapper = Lapper::new(data);
-        lapper
+        Lapper::new(data)
     }
     fn setup_single() -> Lapper<usize, u32> {
         let data: Vec<Iv> = vec![Iv {
@@ -838,8 +841,7 @@ mod tests {
             stop: 35,
             val: 0,
         }];
-        let lapper = Lapper::new(data);
-        lapper
+        Lapper::new(data)
     }
 
     // Test that a query stop that hits an interval start returns no interval
@@ -1207,4 +1209,28 @@ mod tests {
         ]);
         assert_eq!(lapper.count(28974798, 33141355), 1);
     }
+
+    #[test]
+    fn serde_test() {
+        let data = vec![
+            Iv{start:25264912, stop: 25264986, val: 0},
+            Iv{start:27273024, stop: 27273065	, val: 0},
+            Iv{start:27440273, stop: 27440318	, val: 0},
+            Iv{start:27488033, stop: 27488125	, val: 0},
+            Iv{start:27938410, stop: 27938470	, val: 0},
+            Iv{start:27959118, stop: 27959171	, val: 0},
+            Iv{start:28866309, stop: 33141404	, val: 0},
+        ];
+        let lapper = Lapper::new(data);
+
+        let serialized = bincode::serialize(&lapper).unwrap();
+        let deserialzed: Lapper<usize, u32> = bincode::deserialize(&serialized).unwrap();
+
+        let found = deserialzed.find(28974798, 33141355).collect::<Vec<&Iv>>();
+        assert_eq!(found, vec![
+            &Iv{start:28866309, stop: 33141404	, val: 0},
+        ]);
+        assert_eq!(deserialzed.count(28974798, 33141355), 1);
+    }
+
 }
